/******************** (C) COPYRIGHT 2008 STMicroelectronics ********************
* File Name          : main.c
* Author             : MCD Application Team
* Version            : V2.0.3
* Date               : 09/22/2008
* Description        : Main program body.
********************************************************************************
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <intrinsics.h>
#include "stm32f10x_lib.h"
#include "platform_config.h"

#include "utils.h"

#include "arm_comm.h"

#include "lcd-320x240.h"
#include "touchscreen.h"
#include "gps.h"

#if defined(CONFIG_GSM)
  #include "gsm-hw.h"
  #include "gsm.h"
#endif	/* CONFIG_GSM */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
const char *SMS_NUMBER = "+359876041101";

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
GPIO_InitTypeDef GPIO_InitStructure;
ErrorStatus HSEStartUpStatus;
Int32U CriticalSecCntr;

volatile unsigned int counter_1ms;

/* Private function prototypes -----------------------------------------------*/
void RCC_Configuration(void);
void NVIC_Configuration(void);
void Delay(vu32 nCount);

/* Private functions ---------------------------------------------------------*/
static void set_gpio_lowpwr(void);


static void reset_peripherals(void)
{
	RCC_APB2PeriphResetCmd(RCC_APB2Periph_ADC1
				| RCC_APB2Periph_ADC2
				| RCC_APB2Periph_TIM1
				| RCC_APB2Periph_SPI1
				| RCC_APB2Periph_TIM8
				| RCC_APB2Periph_USART1
				| RCC_APB2Periph_ADC3,
				ENABLE);
	RCC_APB1PeriphResetCmd(RCC_APB1Periph_ALL, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ALL, DISABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_ALL, DISABLE);
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1
			| RCC_AHBPeriph_DMA2
			| RCC_AHBPeriph_SRAM
			| RCC_AHBPeriph_FLITF
			| RCC_AHBPeriph_CRC
			| RCC_AHBPeriph_FSMC
			| RCC_AHBPeriph_SDIO,
			DISABLE);
}

static void set_gpio_lowpwr(void)
{
	static const struct {
		GPIO_TypeDef *port;
		unsigned int outm;
		unsigned int outst;
	} gpios[] = {
		{ GPIOA, 0x1fee, 0x0100 },
		{ GPIOB, 0xe023, 0x0000 },
		{ GPIOC, 0xf0ff, 0x0000 },
		{ GPIOD, 0xdff3, 0x00b0 },
		{ GPIOE, 0xffef, 0x000c },
		{ GPIOF, 0xffff, 0x0000 },
		{ GPIOG, 0xffff, 0x0000 },
	};

	size_t i;
	
	for(i=0; i<ARRAY_NUMELEM(gpios); i++) {
		unsigned int pin;
		for(pin=1<<15; pin; pin>>=1) {
			if(gpios[i].outm & pin) {
				GPIO_InitStructure.GPIO_Pin = pin;
				GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
				GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
				GPIO_Init(gpios[i].port, &GPIO_InitStructure);
				if(gpios[i].outst & pin) {
					/* set as high output */
					GPIO_SetBits(gpios[i].port, pin);
				} else {
					/* set as low output */
					GPIO_ResetBits(gpios[i].port, pin);
				}
			} else {
				/* set as input */
				GPIO_InitStructure.GPIO_Pin = pin;
				GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
				GPIO_Init(gpios[i].port, &GPIO_InitStructure);
			}
		}
	}
}


static void shutdown(void)
{
	/* the code here shutdowns EVERYTHING in order to save power */
	LCD_EnterDeepStandby();
	LCD_SetBacklight(0);
	SysTick_ITConfig(DISABLE);
	SysTick_CounterCmd(SysTick_Counter_Disable);

	__disable_interrupt();
	
	NVIC->ICER[0] = NVIC_ICER_CLRENA;
	NVIC->ICER[1] = NVIC_ICER_CLRENA;
	
	//set_all_gpios_to_ain();
	set_gpio_lowpwr();
	
	reset_peripherals();
#if 1
	for(;;) {
		PWR_EnterSTANDBYMode();
	}
#elif 0
	for(;;) {
		PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_WFE);
		__WFE();
		__WFI();
	}
#endif
}

/* Olimex home */
static struct gps_position home_pos = {
	((42*100 + 9) << 14) + (1357 * (1<<14))/60/100,
	'N',
	((24*100 + 46) << 14) + (3707 * (1<<14))/60/100,
	'E'
};
static bool home_pos_valid = true;


static void update_gps_info(void)
{
	struct gps_gprmc info;
	struct gps_gpgga info2;
	long home_distance;

	LCD_ConSetPos(0,0);

	if(!gps_get_gprmc(&info) && !gps_get_gpgga(&info2)) {
		char buf[16];
		if(info.status != 'A') {
			LCD_ConSetColor(LCD_COLOR_BLACK, LCD_COLOR_RED);
		} else {
			LCD_ConSetColor(LCD_COLOR_BLACK, LCD_COLOR_WHITE);
		}
		gps_print_polar(buf, info.pos.latitude);
		LCD_Printf("%c %s\n", info.pos.ns, buf);
		gps_print_polar(buf, info.pos.longitude);
		LCD_Printf("%c %s\n", info.pos.ew, buf);

		LCD_Printf("Time %d:%d:%d\nDate %d.%d.%d\n",
				info.utc_time.hour,
				info.utc_time.min,
				info.utc_time.sec,
				info.utc_date.day,
				info.utc_date.month,
				info.utc_date.year);
		LCD_Printf("Speed=%d.%02d m/s\n",
				gps_real_int(info.speed),
				gps_real_fract100(info.speed));
#if 1
		LCD_Printf("Alt=%dm S=%d\n",
				gps_real_int(info2.msl_altitude),
				info2.sats_used);
#else
		LCD_Printf("%d  %d\n",
			   	gps_calc_course_degr(&info.pos, &home_pos),
				gps_real_int(info.course));
#endif
		if(home_pos_valid) {
			home_distance = gps_calc_distance_m(&home_pos, &info.pos);
		}
		if(!home_pos_valid) {
			LCD_Printf("No home set\n");
		} else if(home_distance < 0) {
			/* at least inform user about our
			 * crippled distance measurement code
			 */
			LCD_Printf("DISTANCE ERR\n");
		} else if(home_distance >= 10000) {
			LCD_Printf("Home %d km\n", home_distance/1000);
		} else {
			LCD_Printf("Home %d m\n", home_distance);
		}
	} else {
		LCD_Printf("GPS not\nconnected\nor no valid\ndata\n\n\n");
	}
}


/*-------------------------------------------------------------------------*/

#if defined(CONFIG_GSM)

static void send_position_with_sms(const char *sms_number)
{
	static char buf[256];
	struct gps_gprmc info;
	struct gps_gpgga info2;

	if(!gps_get_gprmc(&info) && !gps_get_gpgga(&info2)) {
		char lat[16];
		char lon[16];

		gps_print_polar(lat, info.pos.latitude);
		gps_print_polar(lon, info.pos.latitude);
		sprintf(buf, "%c %s, %c %s, Time %d:%d:%d %d.%d.%d, "
				"Speed=%d%d m/s, Altitude=%d m, Sats=%d",
				info.pos.ns, lat,
				info.pos.ew, lon,
				info.utc_time.hour,
				info.utc_time.min,
				info.utc_time.sec,
				info.utc_date.day,
				info.utc_date.month,
				info.utc_date.year,
				gps_real_int(info.speed),
				gps_real_fract100(info.speed),
				gps_real_int(info2.msl_altitude),
				info2.sats_used);
		gsm_send_sms(sms_number, buf);
	}
}


static int gsm_ring_cntr;
static int gsm_num_rings;
static char gsm_clip_phonenum[64];

static void gsm_ring_cb(void)
{
	if(gsm_ring_cntr)
		gsm_num_rings++;
	else
		gsm_num_rings = 1;
	gsm_ring_cntr = 20;
}

static void gsm_clip_cb(const char *phonenum)
{
	if(strlen(phonenum) < (sizeof(gsm_clip_phonenum)-1))
		strcpy(gsm_clip_phonenum, phonenum);
}

struct gsm_notifications gsm_callbacks = {
	.ring = gsm_ring_cb,
	.clip = gsm_clip_cb,
};

static void update_gsm_info(void)
{
	int stat;
	struct gsm_battery_status batstat;

	LCD_ConSetPos(0,0);
	if(!gsm_check_sanity()) {
		LCD_Printf("***GSM*ERROR***\n\n");
	} else {
		LCD_Printf("------GSM------\n");
	}
	if(gsm_ring_cntr) {
		if(strlen(gsm_clip_phonenum)>0) {
			LCD_Printf("RING from:\n%s\n", gsm_clip_phonenum);
		} else {
			LCD_Printf("RINGING...\n");
		}
		gsm_ring_cntr--;
		if(!gsm_ring_cntr) {
			gsm_clip_phonenum[0]='\0';
			gsm_num_rings = 0;
		}
	}

	stat = gsm_get_battery_perc(&batstat);
	if(stat) {
		LCD_Printf("BAT ERR %d\n", stat);
	} else {
		LCD_Printf("BAT =%d mV\n", batstat.voltage_mv);
		if(batstat.bcs == GSM_BAT_CHARGING) {
			LCD_Printf("BAT charging...");
		} else if(batstat.bcs == GSM_BAT_CHARGING_FINISHED) {
			LCD_Printf("BAT charged.");
		} else {
			LCD_Printf("BAT %d%%\n", batstat.bcl);
		}
	}

        stat=gsm_get_signal_quality_report();

	// print current baudrate
	LCD_Printf("BAUD = %dbps\n", gsm_hw_get_baud());
	LCD_Printf("\n\n\n\n\n\n\n");
}

static void check_battery_level(void)
{
	int stat;
	struct gsm_battery_status batstat;

	stat = gsm_get_battery_perc(&batstat);
	if(!stat && (batstat.voltage_mv < 3500)) {
		/* power down to protect battery */
		shutdown();
	}

}

#endif	/* CONFIG_GSM */

/*******************************************************************************
* Function Name  : main
* Description    : Main program.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
  const u16 lcd_brush_colors[] = {
	  LCD_COLOR_BLACK, LCD_COLOR_BLUE, LCD_COLOR_RED, LCD_COLOR_GREEN,
	  LCD_COLOR_WHITE,  LCD_COLOR_GRAY
  };
  enum {
	  ST_GPS_INFO,
	  ST_GSM_INFO,
	  ST_DRAWING,
  } cstate = ST_DRAWING;
  int color_i = 0;
  int prev_pendown = 0;
  int lcd_on = 1;
  int lcd_backlight_on = 1;

#ifdef DEBUG
  debug();
#endif

  ENTR_CRT_SECTION();

  /* System Clocks Configuration **********************************************/
  RCC_Configuration();

  /* NVIC Configuration *******************************************************/
  NVIC_Configuration();

  /* Configure all unused GPIO port pins in Analog Input mode (floating input
     trigger OFF), this will reduce the power consumption and increase the device
     immunity against EMI/EMC *************************************************/
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB |
                         RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD |
                         RCC_APB2Periph_GPIOE | RCC_APB2Periph_GPIOF |
                         RCC_APB2Periph_GPIOG | RCC_APB2Periph_USART1 |
                         RCC_APB2Periph_ADC3, ENABLE);
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_Init(GPIOC, &GPIO_InitStructure);
  GPIO_Init(GPIOD, &GPIO_InitStructure);
  GPIO_Init(GPIOE, &GPIO_InitStructure);
  GPIO_Init(GPIOF, &GPIO_InitStructure);
  GPIO_Init(GPIOG, &GPIO_InitStructure);

  EXT_CRT_SECTION();

  /* SysTick end of count event each 10ms with input clock equal to 9MHz (HCLK/8, default) */
  SysTick_SetReload(9000*10);

  /* Enable SysTick interrupt */
  SysTick_ITConfig(ENABLE);
  SysTick_CounterCmd(SysTick_Counter_Enable);

/*
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB |
                         RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD |
                         RCC_APB2Periph_GPIOE, DISABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF | RCC_APB2Periph_GPIOG, ENABLE);

  GPIO_Init(GPIOF, &GPIO_InitStructure);
  GPIO_Init(GPIOG, &GPIO_InitStructure);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF | RCC_APB2Periph_GPIOG, DISABLE);
*/
  /* Enable the FSMC Clock */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

  /* Initialize LCD */
  LCD_Init();

  if(!LCD_LcdIdOk()) {
	  printf("LCD IS NOT RESPONDING!!!\n");
	  assert(0);
	  for(;;);
  }

  LCD_Clear(LCD_COLOR_BLACK);
  LCD_ConSetColor(LCD_COLOR_BLACK, LCD_COLOR_GRAY);
  LCD_ConSetPos(0,0);

  ts_init();
  gps_init();

#if 0
  LCD_Clear(LCD_COLOR_BLACK);
  stat = ts_calibrate(lcd_tscal_cb);
  if(stat) {
          LCD_ConSetColor(LCD_COLOR_BLACK, LCD_COLOR_RED);
          LCD_ConSetPos(0,0);
          LCD_Printf("TOUCHSCREEN\nERROR");
	  for(;;);
  }
#endif

#if 0
  shutdown();
#endif

  LCD_Printf("Initializing\nsystem\ncomponents...");
#if defined(CONFIG_GSM)
  gsm_init();
#endif	/* CONFIG_GSM */
  LCD_ConSetColor(LCD_COLOR_BLACK, LCD_COLOR_GREEN);

  LCD_Clear(LCD_COLOR_BLUE);

  /* superloop */
  while (1)
  {
	  int x,y;
	  int pressed;
	
	  ts_poll(&x,&y,&pressed);
	  gps_poll();
#if defined(CONFIG_GSM)
	gsm_poll();
	update_gsm_info();
	check_battery_level();
#endif	/* CONFIG_GSM */

#if 0
	  if(pressed) {
		  if(!lcd_on) {
			  lcd_on = 1;
			  LCD_ExitSleep();
			  LCD_SetBacklight(1);
		  }
		  if(cstate == ST_DRAWING) {
			  if(y < 320)
				  LCD_SetPixel(x,y, lcd_brush_colors[color_i]);
		  } else {
			  if(!prev_pendown && (y < 100)) {
				  lcd_backlight_on = !lcd_backlight_on;
				  LCD_SetBacklight(lcd_backlight_on);
			  }
		  }

		  /* "Home" pressed. */
		  if((x > (48*0+10)) && (x < (48*0+38)) && (y >= 325)) {
			  struct gps_gprmc info;
			  if(!gps_get_gprmc(&info)) {
				home_pos = info.pos;
				home_pos_valid = true;
			  }
		  }
			
		  /* "Message" pressed. */
		  if((x > (48*1+10)) && (x < (48*1+38)) && (y >= 325)) {
			  if(!prev_pendown) {
#if defined(CONFIG_GSM)
				  send_position_with_sms(SMS_NUMBER);
#endif	/* CONFIG_GSM */
			  }
			  //printf("Message\n");
		  }
			
		  /* "Address Book" pressed. */
		  if((x > (48*2+10)) && (x < (48*2+38)) && (y >= 325)) {
			  LCD_Clear(LCD_COLOR_BLACK);
			  cstate = ST_GPS_INFO;
		  }
			
		  /* "Phone" pressed. */
		  if((x > (48*3+10)) && (x < (48*3+38)) && (y >= 325)) {
			  //printf("Phone\n");
			  LCD_Clear(LCD_COLOR_BLUE);
			  cstate = ST_GSM_INFO;
		  }
			
		  /* "Music" pressed. */
		  if((x > (48*4+10)) && (x < (48*4+38)) && (y >= 325)) {
			  //printf("Music\n");
			  if(cstate != ST_DRAWING)
				  LCD_Clear(LCD_COLOR_WHITE);
			  cstate = ST_DRAWING;
			  if(!prev_pendown) {
				  if((++color_i) > ARRAY_NUMELEM(lcd_brush_colors))
					  color_i = 0;
			  }
		  }	
	  }
#endif
	
  }
}

/*******************************************************************************
* Function Name  : RCC_Configuration
* Description    : Configures the different system clocks.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RCC_Configuration(void)
{
  /* RCC system reset(for debug purpose) */
  RCC_DeInit();

  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    /* Enable Prefetch Buffer */
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    /* Flash 2 wait state */
    FLASH_SetLatency(FLASH_Latency_2);
 	
    /* HCLK = SYSCLK */
    RCC_HCLKConfig(RCC_SYSCLK_Div1);

    /* PCLK2 = HCLK */
    RCC_PCLK2Config(RCC_HCLK_Div1);

    /* PCLK1 = HCLK/2 */
    RCC_PCLK1Config(RCC_HCLK_Div2);

    /* ADCCLK = PCLK2/4 */
    RCC_ADCCLKConfig(RCC_PCLK2_Div4);


    /* PLLCLK = 8MHz * 9 = 72 MHz */
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    /* Enable PLL */
    RCC_PLLCmd(ENABLE);

    /* Wait till PLL is ready */
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
    {
    }

    /* Select PLL as system clock source */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    /* Wait till PLL is used as system clock source */
    while(RCC_GetSYSCLKSource() != 0x08)
    {
    }
    RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_1Div5);
  }
}

/*******************************************************************************
* Function Name  : NVIC_Configuration
* Description    : Configures Vector Table base location.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void NVIC_Configuration(void)
{
#ifdef  VECT_TAB_RAM
  /* Set the Vector Table base location at 0x20000000 */
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
#endif
   /* Configure the NVIC Preemption Priority Bits */
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);
}

/*******************************************************************************
* Function Name  : Delay
* Description    : Inserts a delay time.
* Input          : nCount: specifies the delay time length.
* Output         : None
* Return         : None
*******************************************************************************/
void Delay(vu32 nCount)
{
  for(; nCount != 0; nCount--);
}

#ifdef  DEBUG
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(u8* file, u32 line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

#if defined(NDEBUG)
__interwork size_t __write(int handle, const unsigned char *buf, size_t size)
{
	(void)handle; (void)buf;
	return size;
}
#endif



